@@ -1,3 +1,4 @@
+import gzip
 from http.server import BaseHTTPRequestHandler, HTTPServer
 import os
 import threading
@@ -7,9 +8,10 @@ import unittest
 import pytest
 
 from prometheus_client import (
-    CollectorRegistry, CONTENT_TYPE_LATEST, core, Counter, delete_from_gateway,
-    Enum, Gauge, generate_latest, Histogram, Info, instance_ip_grouping_key,
-    Metric, push_to_gateway, pushadd_to_gateway, Summary,
+    CollectorRegistry, CONTENT_TYPE_LATEST, CONTENT_TYPE_PLAIN_0_0_4,
+    CONTENT_TYPE_PLAIN_1_0_0, core, Counter, delete_from_gateway, Enum, Gauge,
+    generate_latest, Histogram, Info, instance_ip_grouping_key, Metric,
+    push_to_gateway, pushadd_to_gateway, Summary,
 )
 from prometheus_client.core import GaugeHistogramMetricFamily, Timestamp
 from prometheus_client.exposition import (
@@ -46,8 +48,8 @@ cc_total 1.0
 # HELP cc_created A counter
 # TYPE cc_created gauge
 cc_created 123.456
-""", generate_latest(self.registry))
-        
+""", generate_latest(self.registry, openmetrics.ALLOWUTF8))
+
     def test_counter_utf8(self):
         c = Counter('utf8.cc', 'A counter', registry=self.registry)
         c.inc()
@@ -57,7 +59,18 @@ cc_created 123.456
 # HELP "utf8.cc_created" A counter
 # TYPE "utf8.cc_created" gauge
 {"utf8.cc_created"} 123.456
-""", generate_latest(self.registry))
+""", generate_latest(self.registry, openmetrics.ALLOWUTF8))
+
+    def test_counter_utf8_escaped_underscores(self):
+        c = Counter('utf8.cc', 'A counter', registry=self.registry)
+        c.inc()
+        assert b"""# HELP utf8_cc_total A counter
+# TYPE utf8_cc_total counter
+utf8_cc_total 1.0
+# HELP utf8_cc_created A counter
+# TYPE utf8_cc_created gauge
+utf8_cc_created 123.456
+""" == generate_latest(self.registry, openmetrics.UNDERSCORES)
 
     def test_counter_name_unit_append(self):
         c = Counter('requests', 'Request counter', unit="total", registry=self.registry)
@@ -264,70 +277,70 @@ class TestPushGateway(unittest.TestCase):
         push_to_gateway(self.address, "my_job", self.registry)
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_push_schemeless_url(self):
         push_to_gateway(self.address.replace('http://', ''), "my_job", self.registry)
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_push_with_groupingkey(self):
         push_to_gateway(self.address, "my_job", self.registry, {'a': 9})
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job/a/9')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_push_with_groupingkey_empty_label(self):
         push_to_gateway(self.address, "my_job", self.registry, {'a': ''})
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job/a@base64/=')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_push_with_complex_groupingkey(self):
         push_to_gateway(self.address, "my_job", self.registry, {'a': 9, 'b': 'a/ z'})
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job/a/9/b@base64/YS8geg==')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_push_with_complex_job(self):
         push_to_gateway(self.address, "my/job", self.registry)
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job@base64/bXkvam9i')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_pushadd(self):
         pushadd_to_gateway(self.address, "my_job", self.registry)
         self.assertEqual(self.requests[0][0].command, 'POST')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_pushadd_with_groupingkey(self):
         pushadd_to_gateway(self.address, "my_job", self.registry, {'a': 9})
         self.assertEqual(self.requests[0][0].command, 'POST')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job/a/9')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_delete(self):
         delete_from_gateway(self.address, "my_job")
         self.assertEqual(self.requests[0][0].command, 'DELETE')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'')
 
     def test_delete_with_groupingkey(self):
         delete_from_gateway(self.address, "my_job", {'a': 9})
         self.assertEqual(self.requests[0][0].command, 'DELETE')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job/a/9')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'')
 
     def test_push_with_handler(self):
@@ -340,7 +353,7 @@ class TestPushGateway(unittest.TestCase):
         push_to_gateway(self.address, "my_job", self.registry, handler=my_test_handler)
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][0].headers.get('x-test-header'), 'foobar')
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
@@ -351,7 +364,7 @@ class TestPushGateway(unittest.TestCase):
         push_to_gateway(self.address, "my_job_with_basic_auth", self.registry, handler=my_auth_handler)
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job_with_basic_auth')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_push_with_tls_auth_handler(self):
@@ -362,7 +375,7 @@ class TestPushGateway(unittest.TestCase):
         push_to_gateway(self.address, "my_job_with_tls_auth", self.registry, handler=my_auth_handler)
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job_with_tls_auth')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
     def test_push_with_redirect_handler(self):
@@ -372,7 +385,7 @@ class TestPushGateway(unittest.TestCase):
         push_to_gateway(self.address, "my_job_with_redirect", self.registry, handler=my_redirect_handler)
         self.assertEqual(self.requests[0][0].command, 'PUT')
         self.assertEqual(self.requests[0][0].path, '/metrics/job/my_job_with_redirect')
-        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_LATEST)
+        self.assertEqual(self.requests[0][0].headers.get('content-type'), CONTENT_TYPE_PLAIN_0_0_4)
         self.assertEqual(self.requests[0][1], b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
 
         # ensure the redirect preserved request settings from the initial request.
@@ -392,6 +405,30 @@ class TestPushGateway(unittest.TestCase):
 
         self.assertNotIn('//', self.requests[0][0].path)
 
+    def test_push_with_gzip_compression(self):
+        push_to_gateway(self.address, "my_job", self.registry, compression='gzip')
+        request, body = self.requests[0]
+        self.assertEqual(request.headers.get('content-encoding'), 'gzip')
+        decompressed = gzip.decompress(body)
+        self.assertEqual(decompressed, b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
+
+    def test_push_with_snappy_compression(self):
+        snappy = pytest.importorskip('snappy')
+        push_to_gateway(self.address, "my_job", self.registry, compression='snappy')
+        request, body = self.requests[0]
+        self.assertEqual(request.headers.get('content-encoding'), 'snappy')
+        decompressor = snappy.StreamDecompressor()
+        decompressed = decompressor.decompress(body)
+        flush = getattr(decompressor, 'flush', None)
+        if callable(flush):
+            decompressed += flush()
+        self.assertEqual(decompressed, b'# HELP g help\n# TYPE g gauge\ng 0.0\n')
+
+    def test_push_with_invalid_compression(self):
+        with self.assertRaisesRegex(ValueError, 'Unsupported compression type'):
+            push_to_gateway(self.address, "my_job", self.registry, compression='brotli')
+        self.assertEqual(self.requests, [])
+
     def test_instance_ip_grouping_key(self):
         self.assertTrue('' != instance_ip_grouping_key()['instance'])
 
@@ -423,7 +460,7 @@ class Collector:
 
 def _expect_metric_exception(registry, expected_error):
     try:
-        generate_latest(registry)
+        generate_latest(registry, openmetrics.ALLOWUTF8)
     except expected_error as exception:
         assert isinstance(exception.args[-1], core.Metric)
         # Got a valid error as expected, return quietly
@@ -484,10 +521,251 @@ def test_histogram_metric_families(MetricFamily, registry, buckets, sum_value, e
     _expect_metric_exception(registry, error)
 
 
-def test_choose_encoder():
-    assert choose_encoder(None) == (generate_latest, CONTENT_TYPE_LATEST)
-    assert choose_encoder(CONTENT_TYPE_LATEST) == (generate_latest, CONTENT_TYPE_LATEST)
-    assert choose_encoder(openmetrics.CONTENT_TYPE_LATEST) == (openmetrics.generate_latest, openmetrics.CONTENT_TYPE_LATEST)
+class TestChooseEncoder(unittest.TestCase):
+    def setUp(self):
+        self.registry = CollectorRegistry()
+        c = Counter('dotted.counter', 'A counter', registry=self.registry)
+        c.inc()
+
+    def custom_collector(self, metric_family):
+        class CustomCollector:
+            def collect(self):
+                return [metric_family]
+
+        self.registry.register(CustomCollector())
+
+    def assert_is_escaped(self, exp):
+        self.assertRegex(exp, r'.*\ndotted_counter_total 1.0\n.*')
+
+    def assert_is_utf8(self, exp):
+        self.assertRegex(exp, r'.*\n{"dotted.counter_total"} 1.0\n.*')
+
+    def assert_is_prom(self, exp):
+        self.assertNotRegex(exp, r'# EOF')
+
+    def assert_is_openmetrics(self, exp):
+        self.assertRegex(exp, r'# EOF')
+
+    def test_default_encoder(self):
+        generator, content_type = choose_encoder(None)
+        assert content_type == CONTENT_TYPE_PLAIN_0_0_4
+        exp = generator(self.registry).decode('utf-8')
+        self.assert_is_escaped(exp)
+        self.assert_is_prom(exp)
+
+    def test_plain_encoder(self):
+        generator, content_type = choose_encoder(CONTENT_TYPE_PLAIN_0_0_4)
+        assert content_type == CONTENT_TYPE_PLAIN_0_0_4
+        exp = generator(self.registry).decode('utf-8')
+        self.assert_is_escaped(exp)
+        self.assert_is_prom(exp)
+
+    def test_openmetrics_latest(self):
+        generator, content_type = choose_encoder(openmetrics.CONTENT_TYPE_LATEST)
+        assert content_type == 'application/openmetrics-text; version=1.0.0; charset=utf-8; escaping=underscores'
+        exp = generator(self.registry).decode('utf-8')
+        self.assert_is_escaped(exp)
+        self.assert_is_openmetrics(exp)
+
+    def test_openmetrics_utf8(self):
+        generator, content_type = choose_encoder(openmetrics.CONTENT_TYPE_LATEST + '; escaping=allow-utf-8')
+        assert content_type == openmetrics.CONTENT_TYPE_LATEST + '; escaping=allow-utf-8'
+        exp = generator(self.registry).decode('utf-8')
+        self.assert_is_utf8(exp)
+        self.assert_is_openmetrics(exp)
+
+    def test_openmetrics_dots_escaping(self):
+        generator, content_type = choose_encoder(openmetrics.CONTENT_TYPE_LATEST + '; escaping=dots')
+        assert content_type == openmetrics.CONTENT_TYPE_LATEST + '; escaping=dots'
+        exp = generator(self.registry).decode('utf-8')
+        self.assertRegex(exp, r'.*\ndotted_dot_counter__total 1.0\n.*')
+        self.assert_is_openmetrics(exp)
+
+    def test_prom_latest(self):
+        generator, content_type = choose_encoder(CONTENT_TYPE_LATEST)
+        assert content_type == CONTENT_TYPE_PLAIN_1_0_0 + '; escaping=underscores'
+        exp = generator(self.registry).decode('utf-8')
+        self.assert_is_escaped(exp)
+        self.assert_is_prom(exp)
+
+    def test_prom_plain_1_0_0(self):
+        generator, content_type = choose_encoder(CONTENT_TYPE_PLAIN_1_0_0)
+        assert content_type == CONTENT_TYPE_PLAIN_1_0_0 + '; escaping=underscores'
+        exp = generator(self.registry).decode('utf-8')
+        self.assert_is_escaped(exp)
+        self.assert_is_prom(exp)
+
+    def test_prom_utf8(self):
+        generator, content_type = choose_encoder(CONTENT_TYPE_PLAIN_1_0_0 + '; escaping=allow-utf-8')
+        assert content_type == CONTENT_TYPE_PLAIN_1_0_0 + '; escaping=allow-utf-8'
+        exp = generator(self.registry).decode('utf-8')
+        self.assert_is_utf8(exp)
+        self.assert_is_prom(exp)
+
+    def test_prom_dots_escaping(self):
+        generator, content_type = choose_encoder(CONTENT_TYPE_PLAIN_1_0_0 + '; escaping=dots')
+        assert content_type == CONTENT_TYPE_PLAIN_1_0_0 + '; escaping=dots'
+        exp = generator(self.registry).decode('utf-8')
+        self.assertRegex(exp, r'.*\ndotted_dot_counter__total 1.0\n.*')
+        self.assert_is_prom(exp)
+
+    def test_openmetrics_no_version(self):
+        generator, content_type = choose_encoder('application/openmetrics-text; charset=utf-8; escaping=allow-utf-8')
+        assert content_type == 'application/openmetrics-text; version=1.0.0; charset=utf-8'
+        exp = generator(self.registry).decode('utf-8')
+        # No version -- allow-utf-8 rejected.
+        self.assert_is_escaped(exp)
+        self.assert_is_openmetrics(exp)
+
+    def test_prom_no_version(self):
+        generator, content_type = choose_encoder('text/plain; charset=utf-8; escaping=allow-utf-8')
+        assert content_type == 'text/plain; version=0.0.4; charset=utf-8'
+        exp = generator(self.registry).decode('utf-8')
+        # No version -- allow-utf-8 rejected.
+        self.assert_is_escaped(exp)
+        self.assert_is_prom(exp)
+
+
+@pytest.mark.parametrize("scenario", [
+    {
+        "name": "empty string",
+        "input": "",
+        "expectedUnderscores": "",
+        "expectedDots": "",
+        "expectedValue": "",
+    },
+    {
+        "name": "legacy valid metric name",
+        "input": "no:escaping_required",
+        "expectedUnderscores": "no:escaping_required",
+        "expectedDots": "no:escaping__required",
+        "expectedValue": "no:escaping_required",
+    },
+    {
+        "name": "metric name with dots",
+        "input": "mysystem.prod.west.cpu.load",
+        "expectedUnderscores": "mysystem_prod_west_cpu_load",
+        "expectedDots": "mysystem_dot_prod_dot_west_dot_cpu_dot_load",
+        "expectedValue": "U__mysystem_2e_prod_2e_west_2e_cpu_2e_load",
+    },
+    {
+        "name": "metric name with dots and underscore",
+        "input": "mysystem.prod.west.cpu.load_total",
+        "expectedUnderscores": "mysystem_prod_west_cpu_load_total",
+        "expectedDots": "mysystem_dot_prod_dot_west_dot_cpu_dot_load__total",
+        "expectedValue": "U__mysystem_2e_prod_2e_west_2e_cpu_2e_load__total",
+    },
+    {
+        "name": "metric name with dots and colon",
+        "input": "http.status:sum",
+        "expectedUnderscores": "http_status:sum",
+        "expectedDots": "http_dot_status:sum",
+        "expectedValue": "U__http_2e_status:sum",
+    },
+    {
+        "name": "metric name with spaces and emoji",
+        "input": "label with 😱",
+        "expectedUnderscores": "label_with__",
+        "expectedDots": "label__with____",
+        "expectedValue": "U__label_20_with_20__1f631_",
+    },
+    {
+        "name": "metric name with unicode characters > 0x100",
+        "input": "花火",
+        "expectedUnderscores": "__",
+        "expectedDots": "____",
+        "expectedValue": "U___82b1__706b_",
+    },
+    {
+        "name": "metric name with spaces and edge-case value",
+        "input": "label with \u0100",
+        "expectedUnderscores": "label_with__",
+        "expectedDots": "label__with____",
+        "expectedValue": "U__label_20_with_20__100_",
+    },
+])
+def test_escape_metric_name(scenario):
+    input = scenario["input"]
+
+    got = openmetrics.escape_metric_name(input, openmetrics.UNDERSCORES)
+    assert got == scenario["expectedUnderscores"], f"[{scenario['name']}] Underscore escaping failed"
+
+    got = openmetrics.escape_metric_name(input, openmetrics.DOTS)
+    assert got == scenario["expectedDots"], f"[{scenario['name']}] Dots escaping failed"
+
+    got = openmetrics.escape_metric_name(input, openmetrics.VALUES)
+    assert got == scenario["expectedValue"], f"[{scenario['name']}] Value encoding failed"
+
+
+@pytest.mark.parametrize("scenario", [
+    {
+        "name": "empty string",
+        "input": "",
+        "expectedUnderscores": "",
+        "expectedDots": "",
+        "expectedValue": "",
+    },
+    {
+        "name": "legacy valid label name",
+        "input": "no_escaping_required",
+        "expectedUnderscores": "no_escaping_required",
+        "expectedDots": "no__escaping__required",
+        "expectedValue": "no_escaping_required",
+    },
+    {
+        "name": "label name with dots",
+        "input": "mysystem.prod.west.cpu.load",
+        "expectedUnderscores": "mysystem_prod_west_cpu_load",
+        "expectedDots": "mysystem_dot_prod_dot_west_dot_cpu_dot_load",
+        "expectedValue": "U__mysystem_2e_prod_2e_west_2e_cpu_2e_load",
+    },
+    {
+        "name": "label name with dots and underscore",
+        "input": "mysystem.prod.west.cpu.load_total",
+        "expectedUnderscores": "mysystem_prod_west_cpu_load_total",
+        "expectedDots": "mysystem_dot_prod_dot_west_dot_cpu_dot_load__total",
+        "expectedValue": "U__mysystem_2e_prod_2e_west_2e_cpu_2e_load__total",
+    },
+    {
+        "name": "label name with dots and colon",
+        "input": "http.status:sum",
+        "expectedUnderscores": "http_status_sum",
+        "expectedDots": "http_dot_status__sum",
+        "expectedValue": "U__http_2e_status_3a_sum",
+    },
+    {
+        "name": "label name with spaces and emoji",
+        "input": "label with 😱",
+        "expectedUnderscores": "label_with__",
+        "expectedDots": "label__with____",
+        "expectedValue": "U__label_20_with_20__1f631_",
+    },
+    {
+        "name": "label name with unicode characters > 0x100",
+        "input": "花火",
+        "expectedUnderscores": "__",
+        "expectedDots": "____",
+        "expectedValue": "U___82b1__706b_",
+    },
+    {
+        "name": "label name with spaces and edge-case value",
+        "input": "label with \u0100",
+        "expectedUnderscores": "label_with__",
+        "expectedDots": "label__with____",
+        "expectedValue": "U__label_20_with_20__100_",
+    },
+])
+def test_escape_label_name(scenario):
+    input = scenario["input"]
+
+    got = openmetrics.escape_label_name(input, openmetrics.UNDERSCORES)
+    assert got == scenario["expectedUnderscores"], f"[{scenario['name']}] Underscore escaping failed"
+
+    got = openmetrics.escape_label_name(input, openmetrics.DOTS)
+    assert got == scenario["expectedDots"], f"[{scenario['name']}] Dots escaping failed"
+
+    got = openmetrics.escape_label_name(input, openmetrics.VALUES)
+    assert got == scenario["expectedValue"], f"[{scenario['name']}] Value encoding failed"
 
 
 if __name__ == '__main__':
